function [Accuracy, PredClass, Prob, lGCC] = test_digit_classifier(ccMean,ccVar,img,lab,varargin)
% FORMAT [Accuracy, PredClass, Prob, lGCC] = test_digit_classifier(ccMean,ccVar,img,lab[,VarOpt])
% Predict digit identity for images and compare to known labels lab,
% using a Naive Bayesian Classifier
%
% ccMean - Class conditional means, from training data
% ccVar  - Class conditional variances, from training data
% img    - test data, images of handwriting
% lab    - true labels, to compute accuracy of classification
% VarOpt - Variance option, to control different assumptions on the
%          variance (and deal with 0 variances).  One of
%             0: Pool (average) variance over all classes at each pixel [Default]
%             1: Pool (average) variance over all classes, all pixels
%             2: At a voxel with zero variance, set variance to minium of
%                variance in other classes with non-zero variance
%             3: At a voxel with zero variance, set variance to 1/100-th
%                of the mean variance of non-zero classes
%             4: Globally constrain variance, so smallest variance is at
%                least 1/100-th of the maximal variance
%             5: Pool variance over all classes at each pixel, using the median
%             6: Pool variance over all classes, all pixels, using the median
%
% Assumes equal prevlance of the 10 digits (P(lab)=1/10))
%
% Regularises the variance
%________________________________________________________________________
% Script:   test_digit_classifier
% Purpose:  Test a trained N.B.C with Gaussian data
% Author:   Tom Nichols
% Date:     29 Jan 2012
%

if nargin>4
  VarOpt = varargin{1};
else
  VarOpt = 2;
end


% Constants (no magic numbers!)
nDigit = 10;
nCase  = size(img,2);

PredClass = zeros(1,nCase);
Prob      = zeros(1,nCase);
lGCC      = zeros(nDigit,nCase);

for i = 1:nCase
  lcc          = logGausClassCond(img(:,i),ccMean,ccVar,VarOpt);
  [mlcc Pred]  = max(lcc);  % 2nd return argument from max tells us which
                            % element is maximal! 
  PredClass(i) = Pred;
  Prob(i)      = mlcc;
  lGCC(:,i)    = lcc';         
end

Accuracy = mean(PredClass==lab);

return



function f = logGausClassCond(x,mn,va,VarOpt) 
% FORMAT f = logGausClassCond(x,mn,va,VarOpt) 
% Compute all 10 class conditionals for one image
%
% x  - Column vector of data,   nPixel x 1
% mn - Mean for for each class, nPixel x nDigit
% va - Variance for each class, nPixel x nDigit
% VarOpt - See help at top of file

nDigit = 10;
nPixel = size(x,1);
va_mx  = max(va(:));
va_min = max(va,[],2);

switch VarOpt
 case 0 
  % Pool over classes, at each pixel
  va = repmat(mean(va,2),[1 nDigit]);
 case 1 
  % Pool over all classes & pixels
  va = repmat(mean(va(:)),[nPixel nDigit]);
 case {2,3}
  % Fix only zero variances
  if any(va(:)==0)
    % Find the pixels that have one or more zero-variance classes
    I=find(any(va==0,2))';
    for i=I
      Iz  = find(va(i,:)==0); % zero classes for this pixel
      Inz = find(va(i,:)>0); %  the rest
      if (VarOpt==2)
	va(i,Iz) = min(va(i,Inz));
      else % VarOpt==3
	va(i,Iz) = mean(va(i,Inz))/10^2;
      end
    end
  end 
 case 4
  % Globally constrain the variance, so minimum is no less than 1/100^2 of
  % maximal variance
  va = max(va,va_mx/100^2);
 case 5
  % Pool over classes, at each pixel, using median
  va = repmat(median(va,2),[1 nDigit]);
 case 6 
  % Pool over all classes & pixels, using median
  va = repmat(median(va(:)),[nPixel nDigit]);
 otherwise
  error('Unknown variance option!')
end

xx = repmat(x,[1 10]); % Expand data to be nPixel x nDigit, to facilitate
                       % computations below
f = -sum(log(va))/2 -sum((xx-mn).^2./va)/2;

% Terms absent:  -log(2*pi)*d/2 & log(P(Label)), as both are constant for all x
